@@ -21,8 +21,11 @@ Config management module
 
 __maintainer__ = 'Philippe Normand <philippe@fluendo.com>'
 
+# FIXME: without this line configobj.OPTION_DEFAULTS is None in some tests..
+# EEK!
+from elisa.extern import configobj
+from elisa.extern.configobj import ConfigObj, Section, ConfigObjError
 
-from elisa.extern.configobj import ConfigObj, Section
 from elisa.core import log
 
 import os
@@ -79,8 +82,8 @@ class Config(log.Loggable):
         self.info("Using %r config file", config_file)
         try:
             self._config = ConfigObj(config_file, unrepr=True)
-        except Exception,ex:
-            errors = [ error.msg for error in ex.errors ]
+        except ConfigObjError, ex:
+            errors = [ error.message for error in ex.errors ]
             errors = ';'.join(errors)
             raise ConfigError("Config format error in %s: %s" % (config_file,
                                                                  errors))
