@@ -22,73 +22,78 @@ __maintainer__ = 'Philippe Normand <philippe@fluendo.com>'
 
 from elisa.core import log, config, media_uri
 import os, datetime
-from distutils.version import LooseVersion
+from distutils.version import StrictVersion, LooseVersion
 
 class ConfigUpgrader(log.Loggable):
 
     def __init__(self, current_config, default_config):
         log.Loggable.__init__(self)
         self._current_config = current_config
-        self._current_version = self._get_version(current_config)
         self._default_config = default_config
         self._new_config_fname = 'elisa_new.conf'
         
         today = datetime.date.today().isoformat()
-        self._today = self._current_config.get_option('install_date',
-                                                      section='general',
-                                                      default=today)
+        if self._current_config:
 
-    def _get_version(self, cfg):
-        version = cfg.get_option('version', section='general')
-        if version is None:
-            version = '0.1.7'
+            self._today = self._current_config.get_option('install_date',
+                                                          section='general',
+                                                          default=today)
+            self._current_version = current_config.get_option('version',
+                                                              section='general',
+                                                              default='0.1.7')
+        else:
+            self._today = today
+            self._current_version = '0.1.7'
             
-        cfg_version = LooseVersion(version)
-        return cfg_version
+        self.releases = [(('0.1.7', ), self._update_0_1_to_0_3_1),
+                         (('0.3.1','0.3.1.1', ), self._update_0_3_1_to_0_3_2),
+                         (('0.3.2','0.3.2.1', ), self._update_0_3_2_to_0_3_3),
+                         (('0.3.3','0.3.3.1', '0.3.3.2',
+                           '0.3.4.rc1', '0.3.4.rc2',),
+                          self._update_0_3_3_to_0_3_4),
+                         (('0.3.4','0.3.4.1'), self._update_0_3_4_to_0_3_5),
+                          ]
+        
+    def _get_version_index_upgrade(self, version_str):
+        idx = 0
+        found_idx = 0
+        to_call = None
+        for versions, upgrade_call in self.releases:
+            if version_str in versions:
+                found_idx = idx
+                to_call = upgrade_call
+                break
+            idx += 1
+        return (found_idx, to_call)
     
     def update_for(self, version):
         cfg = self._current_config
         version_str = '.'.join([str(i) for i in version])
+        loose_version = LooseVersion(version_str)
         updated = False
         self.info("Trying to upgrade from %s to %s", self._current_version,
                   version_str)
-        
-        if self._current_version == '0.1.7':
-            cfg = self._update_0_1_to_0_3_1()
+
+        # incrementally upgrade the config throught the releases history to
+        # requested version
+        idx, call = self._get_version_index_upgrade(self._current_version)
+        while idx < len(self.releases) and call != None:
+                   
+            if LooseVersion(self._current_version) >= loose_version:
+                break
+            
+            cfg = call(cfg)
             updated = True
-                
-        if self._current_version == '0.3.1':
-            if version in ((0, 3, 1, 1),(0, 3, 2)):
-                cfg = self._update_0_3_1_to_0_3_2(cfg)
-                updated = True
-                
-        if self._current_version == '0.3.2':
-            if version in ((0, 3, 2, 1),(0, 3, 3)):
-                cfg = self._update_0_3_2_to_0_3_3(cfg)
-                updated = True
-            elif version in ((0, 3, 3, 1), (0, 3, 3, 2),
-                           (0, 3, 4, 'rc1'), (0, 3, 4, 'rc2'),
-                           (0, 3, 4)):
-                cfg = self._update_0_3_2_to_0_3_3(cfg)
-                cfg = self._update_0_3_3_to_0_3_4(cfg)
-                updated = True
-                
-        if self._current_version == '0.3.3':
-            if version in ((0, 3, 3, 1), (0, 3, 3, 2),
-                           (0, 3, 4, 'rc1'), (0, 3, 4, 'rc2'),
-                           (0, 3, 4)):
-                cfg = self._update_0_3_3_to_0_3_4(cfg)
-                updated = True
 
-        if self._current_version == '0.3.4.rc2':
-            if version in ((0, 3, 4),):
-                cfg = self._update_0_3_4_rc2_to_0_3_4(cfg)
-                updated = True
-
-        if self._current_version == '0.3.4':
-            if version in ((0, 3, 4, 1), (0, 3, 5)):
-                cfg = self._update_0_3_4_to_0_3_5(cfg)
-                updated = True
+            # 0.3.4.rc* is considered superior to 0.3.4. So follows a
+            # wrokaround to avoid an infinite loop
+            if self._current_version == '0.3.4':
+               if loose_version < LooseVersion('0.3.5'):
+                   break
+               else:
+                   cfg = self._update_0_3_4_to_0_3_5(cfg)
+                   
+            idx, call = self._get_version_index_upgrade(self._current_version)
                 
         if updated:
             cfg.set_option('install_date', self._today, section='general')
@@ -123,7 +128,7 @@ class ConfigUpgrader(log.Loggable):
             
         full_path = os.path.join(cfg.get_config_dir(),
                                  'elisa_%s.bak' % version)
-        self.info("Doing a backup to the old config on %s", full_path)
+        self.debug("Doing a backup to the old config on %s", full_path)
         cfg.write(full_path)
 
     def _update_0_3_4_to_0_3_5(self, cfg=None):
@@ -133,49 +138,41 @@ class ConfigUpgrader(log.Loggable):
             cfg = self._current_config
 
         service_providers = cfg.get_option('service_providers',
-                                           section='general')
+                                           section='general', default=[])
         if 'updater:updater_service' not in service_providers:
-            service_providers = self._extend_list(service_providers,
+            service_providers = self._extend_list(service_providers, 
                                                   ['updater:updater_service'])
-            cfg.set_option('service_providers', service_providers,
-                           section='general')
 
+        # rename some sections...
+        sections_to_rename = {'base:player_controller': 'player:player_controller',
+                              'poblenou:player_view': 'player:playback_view'
+                              }
+        for old_name, new_name in sections_to_rename.iteritems():
+            cfg.rename_section(old_name, new_name)
+
+        # move visualisation from player_view to player_controller
+        visu = cfg.get_option('visualisation', section='player:playback_view')
+        if visu:
+            cfg.del_option('visualisation', section='player:playback_view')
+            cfg.set_option('visualisation', visu,
+                           section='player:player_controller')
+            
         self._current_version = '0.3.5'
         return cfg
 
-    def _update_0_3_4_rc2_to_0_3_4(self, cfg=None):
-        self.info('Upgrading from 0.3.4.rc2 to 0.3.4')
-        self._backup((0, 3, 4, 'rc2'), cfg)
-        if not cfg:
-            cfg = self._current_config
-
-        # transfer 'start_fullscreen' option from raval:elisa_controller
-        # section to pigment:pigment_context section
-        start_fullscreen = cfg.get_option('start_fullscreen',
-                                          section='raval:elisa_controller')
-        cfg.set_option('start_fullscreen', start_fullscreen,
-                       section='pigment:pigment_context')
-        cfg.del_option('start_fullscreen',
-                       section='raval:elisa_controller')
-
-        service_providers = cfg.get_option('service_providers',
-                                           section='general')
-        if 'updater:updater_service' not in service_providers:
-            service_providers = self._extend_list(service_providers,
-                                                  ['updater:updater_service'])
-            cfg.set_option('service_providers', service_providers,
-                           section='general')
-
-        self._current_version = '0.3.4'
-        return cfg
-
-
     def _update_0_3_3_to_0_3_4(self, cfg=None):
         self.info('Upgrading from 0.3.3 to 0.3.4')
         self._backup((0, 3, 3), cfg)
         if not cfg:
             cfg = self._current_config
 
+        # remove some sections
+        for section_name in ['base:dvd_player_controller',
+                             'poblenou:dvd_player_view',
+                             'dvd:dvd_activity']:
+            cfg.del_section(section_name)
+                             
+
         # remove stage6 media_provider
         media_providers = cfg.get_option('media_providers',
                                          section='general', default=[])
@@ -198,9 +195,11 @@ class ConfigUpgrader(log.Loggable):
         service_providers = self._extend_list(service_providers,
                                               ['media_db:media_scanner',
                                                'osso:osso_service'])
-        if 'updater:updater_service' not in service_providers:
-            service_providers = self._extend_list(service_providers,
-                                                  ['updater:updater_service'])
+
+        # remove the updater service
+        if 'updater:updater_service' in service_providers:
+            service_providers.remove('updater:updater_service')
+        
         cfg.set_option('service_providers', service_providers,
                        section='general')
 
@@ -477,7 +476,7 @@ class ConfigUpgrader(log.Loggable):
         self._current_version = '0.3.2'
         return cfg
             
-    def _update_0_1_to_0_3_1(self):
+    def _update_0_1_to_0_3_1(self, cfg=None):
         self.info('Upgrading from 0.1.x to 0.3.1')
         self._backup()
         cfg = config.Config(self._new_config_fname, self._default_config)
