@@ -121,7 +121,7 @@ from twisted.internet import reactor, defer
 
 from elisa.core import common, manager
 from elisa.core.utils import classinit
-from elisa.base_components import media_provider
+from elisa.core.components import media_provider
 from elisa.core import media_uri
 
 class MediaProviderNotFound(Exception):
@@ -221,7 +221,7 @@ class MediaManager(manager.Manager):
 
         @param uri: The location of the source
         @type uri:  L{elisa.core.media_uri.MediaUri}
-        @rtype:     L{elisa.base_components.media_provider.MediaProvider}
+        @rtype:     L{elisa.core.components.media_provider.MediaProvider}
         """
         provider = None
         providers = self._providers_by_scheme.get(uri.scheme)
@@ -435,7 +435,7 @@ class MediaManager(manager.Manager):
                          of the file on which the event applies to
                          prototype: callable(uri, event)
                          type uri:   L{elisa.core.media_uri.MediaUri}
-                         type event: L{elisa.base_components.media_provider.NotifyEvent}
+                         type event: L{elisa.core.components.media_provider.NotifyEvent}
         """
 
         return self._proxy('monitor_uri', None, uri, callback, *extra_args)
