@@ -20,7 +20,7 @@ Contains locale helper functions
 
 __maintainer__ = 'Benjamin Kampmann <benjamin@fluendo.com>'
 
-import locale, platform
+import locale
 
 
 """
@@ -79,11 +79,7 @@ def system_encoding():
     @return: the encoding used by the system
     """
     
-    if platform.system() == 'Windows':
-        #FIXME : must be dynamic
-        return 'iso8859-1'
-    
-    return 'utf8'
+    return locale.getdefaultlocale()[1]
 
 #if __name__ == '__main__':
 #    print "Reading your locale...."
